==> rview-1.1.8.tgz: package/lib/api.js <==
/**
 * Api endpoint
 * @function api
 * @param {string} moduleName - Name of module
 * @param {object} methods - Method definitions
 * @param {Object} [options] - Optional settings
 * @returns {function[]} - Koa handlers
 */
'use strict'

const { AKoa } = require('akoa')
const co = require('co')

const { spinalcase, camelcase } = require('stringcase')
const { RESERVED_METHODS } = require('../constants')
const renderMiddleware = require('./middlewares/render_middleware')

/** @lends api */
function api (moduleName, module, options = {}) {
  let router = AKoa.newRouter()
  router.head('/', co.wrap(function * knock (ctx) {
    ctx.body = null
  }))
  let $moduleBefore = options.$before
  let $moduleAfter = options.$after
  let $moduleCompile = options.$compile
  let $moduleUrls = options.$urls
  let {
    $urls,
    $compile,
    $before,
    $after,
    $middlewares
  } = module

  let methodNames = Object.keys(module).filter((name) => !/^\$/.test(name))
  let methods = methodNames.reduce((methods, name) => Object.assign(methods, {
    [name]: module[ name ]
  }), {})

  router.use(renderMiddleware({
    name: moduleName,
    methods,
    before: [
      $moduleBefore,
      $before
    ],
    compile: [
      $moduleCompile,
      $compile
    ],
    after: [
      $after,
      $moduleAfter
    ]
  }))

  for (let middleware of $middlewares || []) {
    router.use(middleware)
  }

  let endpoints = {}
  for (let methodName of methodNames) {
    let isReserved = !!~RESERVED_METHODS.split(',').indexOf(methodName)
    if (isReserved) {
      throw new Error(`[rview] Method name "${methodName}" is reserved and you cannot override it.`)
    }
    let isPrivate = /^_/.test(methodName)
    if (isPrivate) {
      // Do not expose private methods
      continue
    }

    let name = [ moduleName, methodName ].join('#')
    let url = ($urls && $urls[ name ]) || ($moduleUrls && $moduleUrls[ name ])
    if (url === false) {
      continue
    }
    if (!url) {
      url = '/'
      for (let name of [ moduleName, methodName ]) {
        if (name !== 'index') {
          url = [ url, spinalcase(name) ].join('/').replace(/\/+/, '/')
        }
      }
    }

    if (endpoints[ url ]) {
      console.warn(`[rview] Deprecated URL detected: ${url}`)
    }
    endpoints[ url ] = name
    router.head(name, url, co.wrap(function * knock (ctx) {
      ctx.body = null
    }))

    router.get(name, url, co.wrap(function * render (ctx) {
      let { params, query } = ctx
      let body = yield ctx.render(methodName, Object.assign({}, query, params))
      ctx.body = body
    }))

    if (camelcase(url) !== spinalcase(url)) {
      router.redirect(camelcase(url), spinalcase(url))
    }
  }

  return Object.assign(router, { endpoints })
}

module.exports = api

==> rview-1.2.0.tgz: package/lib/api.js <==
/**
 * Api endpoint
 * @function api
 * @param {string} moduleName - Name of module
 * @param {object} methods - Method definitions
 * @param {Object} [options] - Optional settings
 * @returns {function[]} - Koa handlers
 */
'use strict'

const { AKoa } = require('akoa')
const co = require('co')

const { spinalcase, camelcase } = require('stringcase')
const { RESERVED_METHODS } = require('../constants')
const renderMiddleware = require('./middlewares/render_middleware')

/** @lends api */
function api (moduleName, module, options = {}) {
  let router = AKoa.newRouter()
  router.head('/', co.wrap(function * knock (ctx) {
    ctx.body = null
  }))
  let $moduleBefore = options.$before
  let $moduleAfter = options.$after
  let $moduleCompile = options.$compile
  let $moduleUrls = options.$urls
  let {
    $urls,
    $compile,
    $before,
    $after,
    $middlewares
  } = module

  let methodNames = Object.keys(module).filter((name) => !/^\$/.test(name))
  let methods = methodNames.reduce((methods, name) => Object.assign(methods, {
    [name]: module[ name ]
  }), {})

  router.use(renderMiddleware({
    name: moduleName,
    methods,
    before: [
      $moduleBefore,
      $before
    ],
    compile: [
      $moduleCompile,
      $compile
    ],
    after: [
      $after,
      $moduleAfter
    ]
  }))

  for (let middleware of $middlewares || []) {
    router.use(middleware)
  }

  let endpoints = {}
  for (let methodName of methodNames) {
    let isReserved = !!~RESERVED_METHODS.split(',').indexOf(methodName)
    if (isReserved) {
      throw new Error(`[rview] Method name "${methodName}" is reserved and you cannot override it.`)
    }
    let isPrivate = /^_/.test(methodName)
    if (isPrivate) {
      // Do not expose private methods
      continue
    }

    let name = [ moduleName, methodName ].join('#')
    let url = ($urls && $urls[ name ]) || ($moduleUrls && $moduleUrls[ name ])
    if (url === false) {
      continue
    }
    if (!url) {
      url = '/'
      for (let name of [ moduleName, methodName ]) {
        if (name !== 'index') {
          url = [ url, spinalcase(name) ].join('/').replace(/\/+/, '/')
        }
      }
    }

    if (endpoints[ url ]) {
      console.warn(`[rview] Deprecated URL detected: ${url}`)
    }
    endpoints[ url ] = name
    router.head(name, url, co.wrap(function * knock (ctx) {
      ctx.body = null
    }))

    router.get(name, url, co.wrap(function * render (ctx) {
      let { params, query } = ctx
      try {
        ctx.body = yield ctx.render(methodName, Object.assign({}, query, params))
      } catch (err) {
        if (err.status) {
          ctx.status = err.status
          ctx.body = Object.assign({}, err)
          return
        }
        throw err
      }
    }))

    if (camelcase(url) !== spinalcase(url)) {
      router.redirect(camelcase(url), spinalcase(url))
    }
  }

  return Object.assign(router, { endpoints })
}

module.exports = api
